import {SimpleGamesystem} from "../../gamesystems/SimpleGamesystem";
import {SimpleState} from "../../gamesystems/states/SimpleState";
import {SimpleTransition} from "../../gamesystems/transitions/SimpleTransition";
import {ScriptAccount} from "../../scriptAccounts/ScriptAccount";
import {ScriptAccountCondition} from "../../gamesystems/conditions/ScriptAccountCondition";

export class SimpleGamesystemParser {

  static parseSimpleGamesystem(jsonObject: any, scriptAccounts: ScriptAccount[] = []) : SimpleGamesystem {
    const gamesystemName = jsonObject.componentName;
    const gamesystemDescription = jsonObject.componentDescription;
    const simpleStates = SimpleGamesystemParser.parseSimpleStates(jsonObject, scriptAccounts)
    const simpleTransitions = SimpleGamesystemParser.parseSimpleTransitions(jsonObject, simpleStates);

    const gamesystem = new SimpleGamesystem(gamesystemName, gamesystemDescription);
    gamesystem.states = simpleStates;
    gamesystem.transitions = simpleTransitions;

    return gamesystem;
  }

  static parseSimpleStates(jsonObject: any, scriptAccounts: ScriptAccount[] = []): SimpleState[] {
    const states: SimpleState[] = [];
    for(let i=0; i<jsonObject.states.length; i++) {
      const state = new SimpleState(jsonObject.states[i].stateLabel,  jsonObject.states[i].stateDescription);
      const conditions = jsonObject.states[i].conditions
      for(let j=0; j<conditions.length; j++) {
        const searchedScriptAccount = scriptAccounts.find(scriptAccount => scriptAccount.componentName === conditions[j].scriptAccount)
        if(searchedScriptAccount != undefined) {
          const scriptAccountCondition =
            ScriptAccountCondition.constructScriptAccountCondition(searchedScriptAccount, conditions[j].minValue, conditions[j].maxValue)

          state.conditions.push(scriptAccountCondition!)
        }
      }

      states.push(state);
    }
    return states;
  }



  static parseSimpleTransitions(jsonObject: any, states: SimpleState[]): SimpleTransition[] {
    const transitions: SimpleTransition[] = [];
    for(let i=0; i<jsonObject.transitions.length; i++) {
      const startingStateLabel = jsonObject.transitions[i].startingState;
      const endingStateLabel = jsonObject.transitions[i].endingState;

      const startingState = states.find(state => state.stateLabel === startingStateLabel);
      const endingState = states.find(state => state.stateLabel === endingStateLabel);

      if(startingState != undefined && endingState != undefined) {
        transitions.push(new SimpleTransition(startingState, endingState));
      } else {
        console.error("Starting or Ending State are not defined!", startingState, endingState)
      }
    }
    return transitions;
  }
}
